@@ -1,14 +1,13 @@
 package Dancer2::Plugin::REST;
 our $AUTHORITY = 'cpan:SUKRIA';
 # ABSTRACT: A plugin for writing RESTful apps with Dancer2
-$Dancer2::Plugin::REST::VERSION = '0.27';
+$Dancer2::Plugin::REST::VERSION = '1.00';
 use strict;
 use warnings;
 
-use Carp 'croak';
+use Carp;
 
 use Dancer2::Plugin;
-use Class::Load qw/ try_load_class /;
 
 # [todo] - add XML support
 my $content_types = {
@@ -16,10 +15,22 @@ my $content_types = {
     yml  => 'text/x-yaml',
 };
 
-register prepare_serializer_for_format => sub {
-    my $dsl = shift;
+has '+app' => (
+    handles => [qw/
+        add_hook
+        add_route
+        setting
+        response
+        request
+        send_error
+        set_response
+    /],
+);
 
-    my $conf        = plugin_setting;
+sub prepare_serializer_for_format :PluginKeyword {
+    my $self = shift;
+
+    my $conf        = $self->config;
     my $serializers = (
         ($conf && exists $conf->{serializers})
         ? $conf->{serializers}
@@ -29,44 +40,39 @@ register prepare_serializer_for_format => sub {
         }
     );
 
-    $dsl->hook(
-        'before' => sub {
-            my $format = $dsl->params->{'format'};
-            $format  ||= $dsl->captures->{'format'} if $dsl->captures;
+    $self->add_hook( Dancer2::Core::Hook->new(
+        name => 'before',
+        code => sub {
+            my $format = $self->request->params->{'format'};
+            $format  ||= $self->request->captures->{'format'} if $self->request->captures;
 
-            unless  ( defined $format ) { 
-                delete $dsl->app->response->{serializer};
-                return;
-            }
+            return delete $self->response->{serializer}
+                unless defined $format;
 
-            my $serializer = $serializers->{$format};
-            
-            unless( $serializer ) {
-                return $dsl->send_error("unsupported format requested: " . $format, 404);
-            }
+            my $serializer = $serializers->{$format}
+                or return $self->send_error("unsupported format requested: " . $format, 404);
 
-            $dsl->set(serializer => $serializer);
-            $dsl->app->set_response( Dancer2::Core::Response->new(
-                %{ $dsl->app->response },
-                serializer => $dsl->set('serializer'),
+            $self->setting(serializer => $serializer);
+
+            $self->set_response( Dancer2::Core::Response->new(
+                %{ $self->response },
+                serializer => $self->setting('serializer'),
             ) );
 
-            my $ct = $content_types->{$format} || $dsl->setting('content_type');
-            $dsl->content_type($ct);
+            $self->response->content_type(
+                $content_types->{$format} || $self->setting('content_type')
+            );
         }
-    );
+    ) );
 };
 
-register resource => sub {
-    my $dsl = shift;
-
-    my ($resource, %triggers) = @_;
+sub resource :PluginKeyword {
+    my ($self, $resource, %triggers) = @_;
 
     my %actions = (
-        get    => 'get',
         update => 'put',
         create => 'post',
-        delete => 'delete',
+        map { $_ => $_ } qw/ get delete /
     );
 
     croak "resource should be given with triggers"
@@ -74,7 +80,7 @@ register resource => sub {
              and grep { $triggers{$_} } keys %actions;
 
     while( my( $action, $code ) = each %triggers ) {
-            $dsl->app->add_route( 
+            $self->add_route( 
                 method => $actions{$action},
                 regexp => $_,
                 code   => $code,
@@ -83,15 +89,14 @@ register resource => sub {
     }
 };
 
-register send_entity => sub {
-    my ($dsl, $entity, $http_code) = @_;
+sub send_entity :PluginKeyword {
+    my ($self, $entity, $http_code) = @_;
 
-    $http_code ||= 200;
-
-    $dsl->status($http_code);
+    $self->response->status($http_code || 200);
     $entity;
 };
 
+# TODO refactor that if my patch goes for Dancer2::Core::HTTP
 my %http_codes = (
 
     # 1xx
@@ -159,22 +164,19 @@ my %http_codes = (
     509 => 'Bandwidth Limit Exceeded',
 );
 
-for my $code (keys %http_codes) {
-    my $helper_name = lc($http_codes{$code});
+plugin_keywords map {
+    my $code = $_;
+    my $helper_name = lc($http_codes{$_});
     $helper_name =~ s/[^\w]+/_/gms;
     $helper_name = "status_${helper_name}";
 
-    register $helper_name => sub {
-        my $dsl = shift;
-
-        $dsl->send_entity(
-            ( $code >= 400 ? {error => $_[0]} : $_[0] ),
+    $helper_name => sub {
+        $_[0]->send_entity(
+            ( $code >= 400 ? {error => $_[1]} : $_[1] ),
             $code
         );
     };
-}
-
-register_plugin for_versions => [2];
+} keys %http_codes;
 
 1;
 
@@ -190,7 +192,7 @@ Dancer2::Plugin::REST - A plugin for writing RESTful apps with Dancer2
 
 =head1 VERSION
 
-version 0.27
+version 1.00
 
 =head1 DESCRIPTION
 
